;
; Example of utilization of the Saclay for ISW detection
; We suppose here that the maps have already been inpainted
;
; AUTHOR: Francois-Xavier Dupe
; DATE: 01/09/2010
; REVISION:
;

pro isw_example

  ;; 1 - Get the maps (here the inpainted maps)
  cmbobs = rims('gmca_wmap7yr_inp.fits') / (2.728 * 1e3)
  galobs = rims('2mass_inpaint_512.fits') / 1.27 ; here 1.27 is the bias between matter and galaxy densities

  ;; 2 - Get the theoretical power spectra
  restore,'2MASSpowerSpec.sav',/verb

  ;; 3 - Apply Saclay method on data
  isw_saclayMethod,cmbobs,galobs,iswspec,cmbspec=cmbspec,galspec=galspec,lmin=3,lmax=100

  ;; 4 - Use the wavelets (Santander way of doing things), be careful it can take a long time
  scales = [50.0, 75.0, 100.0, 150.0, 200.0, 250.0, 300.0, 400.0, 500.0, 600.0, 750.0, 900.0, 1050.0]
  t0 = isw_corrTest(cmbobs,galobs,method=2,lmin=lmin,lmax=lmax,bootstrap=1000,btstpvar=1001,bootmet=1,cmbspec=cmbspec,scales=scales,/nested,/noverb)
  print,'First step: correlation detection'
  print,'p-value =',t0.pvalue
  print,'sigma = ',inverf(1.0d - t0.pvalue)*sqrt(2.0d)


end
