pro test_data_sph, data, masque, mask_on = mask_on

;data : filenames of the data maps (output)
;masque : filename of the mask used (output is valid if mask_on is set)
;mask_on : if set, a mask is applied on the data maps

;one may generate maps with a gap or without a gap.
;then one may process either considering that there is a mask or that there isn't a mask
;the two aspects are independent matter.

;------------------------------------------------------------
;building a mask 
;------------------------------------------------------------
if keyword_set(mask_on) then begin 
		
		if 0 then begin		;either generate another mask ....
		
		print, 'Building the mask map and saving it to disk'
		print, 'Ones in the mask indicate valid pixels, zeros indicate invalid pixels'
		
		read_fits_map, 'wmapmask_nested.fits', mask
		;mask0 = not ( (byte( mask) AND 64)/64. or(byte( mask) AND 8)/8. )		;for instance
		mask0 = not (  (byte( mask) AND 4)/4. )									;for instance
	
		write_fits_map, 'themask.fits', mask0, /nested
		ud_grade, 'themask.fits', 'themask_256.fits', nside = 256
		masque = 'themask_256.fits'
		;masque = 'themask.fits'
		read_fits_map, masque, mask0
		
		endif else begin	;... or use existing mask.
			;masque = 'themask_256.fits'
			masque = 'themask.fits'
			read_fits_map, masque, mask0
		endelse
		
		
endif 
;------------------------------------------------------------



;------------------------------------------------------------
;generating simulated data maps from maps of separated physical components
;
;the observation maps obtained simulate the planck-hfi with three sources and 6 channels
;No beam effect is considered
;------------------------------------------------------------

	print, 'Simulating data maps : mixing three components in 6 channels.'

	A = 0.*dblarr( 6, 3)
	A(*,0) = [1.0, 1.0, 1.0, 1.0, 1.0, 1.0]									;cmb
	A(*,1) = [1.0, 2.20, 7.16, 56.96, 1099.42, 146900.58]					;dust
	A(*,2) = [-1.5102256, -1.0458486, 0.0, 2.2166472, 5.5583054, 11.0261560]  ;sz

	print, 'The mixing matrix used :'
	print, A

	noisescale = dblarr(6,1)
	noisescale = [2.65e-6, 2.33e-6, 3.44e-6, 1.05e-5, 1.07e-4, 4.84e-3]
	ratio2nominal = 1. ;1./2.
	
	print, 'The nominal standard deviations of the instrumental noise in the six channels :'
	print, noisescale
	print, 'The global noise attenuation factor used :'
	print, ratio2nominal
	
	S   = strarr(3)
	S(0) = 'cmb_nested.fits'
	S(1) = 'dust_nested.fits'
	S(2) = 'sz_nested.fits'
	;S(0) = 'cmb_nested_256.fits'
	;S(1) = 'dust_nested_256.fits'
	;S(2) = 'sz_nested_256.fits'
	
	print, 'The file names of the component maps used :'
	print, S

	;mixing and saving to disk
	;ranger les noms des cartes de melanges dans data
	data = strarr(6)
	sensormap = 'sensormap' 
	
		if 1 then begin						;generate new mixtures ... 
		for num_melange = 0, 5 do begin
	
				read_fits_map, S(0), carte_observee 
				carte_observee = A(num_melange, 0 ) *(carte_observee- mean(carte_observee))
				read_fits_map, S(1), carte_obs
				carte_observee = carte_observee + A(num_melange, 1 ) * (carte_obs - mean(carte_obs) )
				read_fits_map, S(2), carte_obs
				carte_observee = carte_observee + A(num_melange, 2 ) * (carte_obs - mean(carte_obs) )
							
				carte_observee = carte_observee + ratio2nominal * noisescale(num_melange) *randomn(seed, (size(carte_observee))(1) )	
				
				if keyword_set(mask_on) then carte_observee = carte_observee * mask0

				data(num_melange) = strcompress(sensormap + string(num_melange+1) + '.fits',  /remove_all)  
				write_fits_map, data(num_melange), carte_observee, /nested
				;mollview, data(num_melange)
		endfor
		endif
	
		if 0 then begin						; ... or use existing mixtures.
		for num_melange = 0, 5 do begin
					data(num_melange) = strcompress(sensormap + string(num_melange+1) + '.fits',  /remove_all)  
		endfor
		endif

	

DONE:

return
end 
