# - Try to find HEALPIX
# Once done this will define
#  HEALPIX_FOUND - System has Healpix installed
#  HEALPIX_INCLUDE_DIRS - The Healpix include directories
#  HEALPIX_LIBRARIES - The libraries needed to use Healpix

if (DEFINED ENV{HEALPIX})
  #HEALPIX is set up in this environment
  set(HEALPIX_PATH $ENV{HEALPIX}/src/cxx/$ENV{HEALPIX_TARGET})

  #FINDING HEADER PATH
  find_path(HEALPIX_INCLUDE_DIR healpix_map.h PATHS ${HEALPIX_PATH} PATH_SUFFIXES include NO_DEFAULT_PATH)

  #FINDING LIBRARIES
  find_library(HEALPIX_CXX NAMES  libhealpix_cxx.a PATHS ${HEALPIX_PATH} PATH_SUFFIXES lib NO_DEFAULT_PATH)
  find_library(CXX_SUPPORT NAMES  libcxxsupport.a  PATHS ${HEALPIX_PATH} PATH_SUFFIXES lib NO_DEFAULT_PATH)
  find_library(FFTPACK     NAMES  libfftpack.a     PATHS ${HEALPIX_PATH} PATH_SUFFIXES lib NO_DEFAULT_PATH)
  find_library(C_UTILS     NAMES  libc_utils.a     PATHS ${HEALPIX_PATH} PATH_SUFFIXES lib NO_DEFAULT_PATH)


  # TRY TO FIND PSHT AND SHARP
  find_library(PSHT  NAMES libpsht.a  PATHS ${HEALPIX_PATH} PATH_SUFFIXES lib NO_DEFAULT_PATH)
  find_library(SHARP NAMES libsharp.a PATHS ${HEALPIX_PATH} PATH_SUFFIXES lib NO_DEFAULT_PATH)

  
  #Exporting libraries and includes
  if(SHARP MATCHES "SHARP-NOTFOUND")
    set(HEALPIX_LIBRARIES ${HEALPIX_CXX} ${CXX_SUPPORT} ${PSHT} ${FFTPACK} ${C_UTILS})
  else()
    set(HEALPIX_LIBRARIES ${HEALPIX_CXX} ${CXX_SUPPORT} ${SHARP} ${FFTPACK} ${C_UTILS})
  endif()

  set(HEALPIX_INCLUDE_DIRS  ${HEALPIX_INCLUDE_DIR})

endif (DEFINED ENV{HEALPIX})


include(FindPackageHandleStandardArgs)
# handle the QUIETLY and REQUIRED arguments and set LIBXML2_FOUND to TRUE
# if all listed variables are TRUE
find_package_handle_standard_args(Healpix  DEFAULT_MSG
                                  HEALPIX_LIBRARIES HEALPIX_INCLUDE_DIRS)

mark_as_advanced(HEALPIX_INCLUDE_DIR HEALPIX_CXX CXX_SUPPORT FFTPACK C_UTILS PSHT SHARP)